@@ -137,17 +137,17 @@ purple_certificate_copy(PurpleCertificate *crt)
 GList *
 purple_certificate_copy_list(GList *crt_list)
 {
-	GList *new, *l;
+	GList *new_l, *l;
 
 	/* First, make a shallow copy of the list */
-	new = g_list_copy(crt_list);
+	new_l = g_list_copy(crt_list);
 
 	/* Now go through and actually duplicate each certificate */
-	for (l = new; l; l = l->next) {
+	for (l = new_l; l; l = l->next) {
 		l->data = purple_certificate_copy(l->data);
 	}
 
-	return new;
+	return new_l;
 }
 
 void
@@ -748,9 +748,9 @@ x509_ca_init(void)
 # ifdef SSL_CERTIFICATES_DIR
 		x509_ca_paths = g_list_append(NULL, g_strdup(SSL_CERTIFICATES_DIR));
 # else
-		x509_ca_paths = g_list_append(NULL, g_build_filename(DATADIR,
-						   "purple", "ca-certs", NULL));
 # endif
+		x509_ca_paths = g_list_append(x509_ca_paths,
+			g_build_filename(DATADIR, "purple", "ca-certs", NULL));
 #endif
 	}
 
@@ -1259,6 +1259,7 @@ x509_tls_cached_cert_in_cache(PurpleCertificateVerificationRequest *vrq)
 				   "to cert_changed.\n");
 		/* vrq now becomes the problem of cert_changed */
 		x509_tls_cached_peer_cert_changed(vrq);
+		return;
 	}
 
 	/* Now get SHA1 sums for both and compare them */
@@ -1545,31 +1546,11 @@ purple_certificate_init(void)
 void
 purple_certificate_uninit(void)
 {
-	GList *full_list, *l;
-
-	/* Unregister all Schemes */
-	full_list = g_list_copy(cert_schemes); /* Make a working copy */
-	for (l = full_list; l; l = l->next) {
-		purple_certificate_unregister_scheme(
-			(PurpleCertificateScheme *) l->data );
-	}
-	g_list_free(full_list);
-
 	/* Unregister all Verifiers */
-	full_list = g_list_copy(cert_verifiers); /* Make a working copy */
-	for (l = full_list; l; l = l->next) {
-		purple_certificate_unregister_verifier(
-			(PurpleCertificateVerifier *) l->data );
-	}
-	g_list_free(full_list);
+	g_list_foreach(cert_verifiers, (GFunc)purple_certificate_unregister_verifier, NULL);
 
 	/* Unregister all Pools */
-	full_list = g_list_copy(cert_pools); /* Make a working copy */
-	for (l = full_list; l; l = l->next) {
-		purple_certificate_unregister_pool(
-			(PurpleCertificatePool *) l->data );
-	}
-	g_list_free(full_list);
+	g_list_foreach(cert_pools, (GFunc)purple_certificate_unregister_pool, NULL);
 }
 
 gpointer
@@ -1897,10 +1878,13 @@ purple_certificate_display_x509(PurpleCertificate *crt)
 
 	/* Make messages */
 	secondary = g_strdup_printf(_("Common name: %s\n\n"
-				      "Fingerprint (SHA1): %s\n\n"
-				      "Activation date: %s\n"
-				      "Expiration date: %s\n"),
-				    cn, sha_asc, activ_str, expir_str);
+								  "Fingerprint (SHA1): %s\n\n"
+								  "Activation date: %s\n"
+								  "Expiration date: %s\n"),
+								cn ? cn : "(null)",
+								sha_asc ? sha_asc : "(null)",
+								activ_str ? activ_str : "(null)",
+								expir_str ? expir_str : "(null)");
 
 	/* Make a semi-pretty display */
 	purple_notify_info(
