@@ -23,6 +23,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  */
+#define _PURPLE_NOTIFY_C_
+
 #include "internal.h"
 #include "dbus-maybe.h"
 #include "notify.h"
@@ -628,6 +630,17 @@ purple_notify_user_info_add_section_header(PurpleNotifyUserInfo *user_info, cons
 	user_info->user_info_entries = g_list_append(user_info->user_info_entries, entry);
 }
 
+void
+purple_notify_user_info_prepend_section_header(PurpleNotifyUserInfo *user_info, const char *label)
+{
+	PurpleNotifyUserInfoEntry *entry;
+	
+	entry = purple_notify_user_info_entry_new(label, NULL);
+	entry->type = PURPLE_NOTIFY_USER_INFO_ENTRY_SECTION_HEADER;
+	
+	user_info->user_info_entries = g_list_prepend(user_info->user_info_entries, entry);
+}
+
 void
 purple_notify_user_info_add_section_break(PurpleNotifyUserInfo *user_info)
 {
@@ -639,6 +652,17 @@ purple_notify_user_info_add_section_break(PurpleNotifyUserInfo *user_info)
 	user_info->user_info_entries = g_list_append(user_info->user_info_entries, entry);
 }
 
+void
+purple_notify_user_info_prepend_section_break(PurpleNotifyUserInfo *user_info)
+{
+	PurpleNotifyUserInfoEntry *entry;
+	
+	entry = purple_notify_user_info_entry_new(NULL, NULL);
+	entry->type = PURPLE_NOTIFY_USER_INFO_ENTRY_SECTION_BREAK;
+	
+	user_info->user_info_entries = g_list_prepend(user_info->user_info_entries, entry);
+}
+
 void
 purple_notify_user_info_remove_last_item(PurpleNotifyUserInfo *user_info)
 {
