@@ -38,6 +38,7 @@ typedef struct _MsnSession MsnSession;
 #include "cmdproc.h"
 #include "nexus.h"
 #include "httpconn.h"
+#include "oim.h"
 
 #include "userlist.h"
 #include "sync.h"
@@ -93,17 +94,22 @@ struct _MsnSession
 	gboolean http_method;
 
 	MsnNotification *notification;
-	MsnNexus *nexus;
-	MsnSync *sync;
-
-	MsnUserList *userlist;
+	MsnNexus        *nexus;
+	MsnOim          *oim;
+	MsnSync         *sync;
+	MsnUserList     *userlist;
+	char            *abch_cachekey;
 
 	int servconns_count; /**< The count of server connections. */
 	GList *switches; /**< The list of all the switchboards. */
-	GList *directconns; /**< The list of all the directconnections. */
 	GList *slplinks; /**< The list of all the slplinks. */
 
-	int conv_seq; /**< The current conversation sequence number. */
+	/*psm info*/
+	char *psm;
+
+#if 0
+	char *blocked_text;
+#endif
 
 	struct
 	{
@@ -111,11 +117,15 @@ struct _MsnSession
 		char *sid;
 		char *mspauth;
 		unsigned long sl;
-		char *file;
 		char *client_ip;
 		int client_port;
-
+		char *mail_url;
+		gulong mail_timestamp;
+		gboolean email_enabled;
 	} passport_info;
+
+	GHashTable *soap_table;
+	guint soap_cleanup_handle;
 };
 
 /**
@@ -224,4 +234,8 @@ void msn_session_set_login_step(MsnSession *session, MsnLoginStep step);
  */
 void msn_session_finish_login(MsnSession *session);
 
+/*post message to User*/
+void msn_session_report_user(MsnSession *session,const char *passport,
+							const char *msg,PurpleMessageFlags flags);
+
 #endif /* _MSN_SESSION_H_ */
