@@ -68,6 +68,7 @@ typedef struct _MsnSoapConnection {
 
 	GQueue *queue;
 	MsnSoapRequest *current_request;
+	gboolean unsafe_debug;
 } MsnSoapConnection;
 
 static gboolean msn_soap_connection_run(gpointer data);
@@ -79,6 +80,7 @@ msn_soap_connection_new(MsnSession *session, const char *host)
 	conn->session = session;
 	conn->host = g_strdup(host);
 	conn->queue = g_queue_new();
+	conn->unsafe_debug = g_getenv("PURPLE_MSN_UNSAFE_DEBUG") != NULL;
 	return conn;
 }
 
@@ -340,12 +342,14 @@ msn_soap_handle_body(MsnSoapConnection *conn, MsnSoapMessage *response)
 	}
 
 	if (fault || body) {
-		MsnSoapRequest *request = conn->current_request;
-		conn->current_request = NULL;
-		request->cb(request->message, response,
-			request->cb_data);
+		if (conn->current_request) {
+			MsnSoapRequest *request = conn->current_request;
+			conn->current_request = NULL;
+			request->cb(request->message, response,
+				request->cb_data);
+			msn_soap_request_destroy(request, FALSE);
+		}
 		msn_soap_message_destroy(response);
-		msn_soap_request_destroy(request, FALSE);
 	}
 
 	return TRUE;
@@ -504,12 +508,10 @@ msn_soap_read_cb(gpointer data, gint fd, PurpleInputCondition cond)
 	if (cnt < 0 && perrno != EAGAIN)
 		purple_debug_info("soap", "read: %s\n", g_strerror(perrno));
 
-#ifndef MSN_UNSAFE_DEBUG
-	if (conn->current_request && conn->current_request->secure)
+	if (conn->current_request && conn->current_request->secure &&
+		!conn->unsafe_debug)
 		purple_debug_misc("soap", "Received secure request.\n");
-	else
-#endif
-	if (count != 0)
+	else if (count != 0)
 		purple_debug_misc("soap", "current %s\n", conn->buf->str + cursor);
 
 	/* && count is necessary for Adium, on OS X the last read always
@@ -657,12 +659,10 @@ msn_soap_connection_run(gpointer data)
 			g_string_append(conn->buf, "\r\n");
 			g_string_append(conn->buf, body);
 
-#ifndef MSN_UNSAFE_DEBUG
-			if (req->secure)
+			if (req->secure && !conn->unsafe_debug)
 				purple_debug_misc("soap", "Sending secure request.\n");
 			else
-#endif
-			purple_debug_misc("soap", "%s\n", conn->buf->str);
+				purple_debug_misc("soap", "%s\n", conn->buf->str);
 
 			conn->handled_len = 0;
 			conn->current_request = req;
